/* hexmerge.c
 *
 * Utility to merge selected address ranges of two hex files into a new hex file
 *
 * Typical use: update a hex file as produced by a compiler with calibration data or
 * similar from a reference hex file (possibly read out of the chip earlier)
 *
 * Note: this program is currently limited to max. 64KB hex files
 *
 * Copyright 2007 by Tom Torfs - IMEC, Belgium
 *
 * Makes use of Libhexfile by David Sullins, slightly modified to ignore type 3 records
 *
 * hex2txt is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, version 2 of the License.
 * 
 * hex2txt is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with hexmerge; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "hexfile.h"

#define MAX_SIZE 65536

int main(int argc, char *argv[])
{
	struct hexfile hf1,hf2;
	FILE *tf;
	unsigned startaddr1, endaddr1, startaddr2, endaddr2;
	unsigned addr, pos;
	unsigned size;
	unsigned char empty;
	unsigned char checksum;
	unsigned char linedata[0x10];

	printf("hexmerge - merge (parts of) two Intel hex/a43 files (max. %d bytes)\n", MAX_SIZE);
	
	if (argc!=6
		|| sscanf(argv[2], "%x:%x", &startaddr1,&endaddr1)!=2 
		|| startaddr1>0xFFFF|| endaddr1>0xFFFF || endaddr1<startaddr1
		|| sscanf(argv[4], "%x:%x", &startaddr2,&endaddr2)!=2 
		|| startaddr2>0xFFFF|| endaddr2>0xFFFF || endaddr2<startaddr2)
	{
		printf("usage: hexmerge <hexfile1> <start>:<end> <hexfile2> <start>:<end> <targetfile>\n");
		printf("<start>:<end> addresses are in hexadecimal and are inclusive\n");
		printf("in case of range overlap, the data from hexfile2 will be used\n");
		printf("example: hexmerge file.hex 0:FFFF reffile.hex F000:FFDF newfile.hex\n");
		return 1;
	}

	if (init_hexfile(&hf1, MAX_SIZE, NULL, argv[1])!=0)
	{
		printf("error opening hex file %s\n", argv[1]);
		return 1;
	}
 
	if (init_hexfile(&hf2, MAX_SIZE, NULL, argv[3])!=0)
	{
		printf("error opening hex file %s\n", argv[2]);
		destroy_hexfile(&hf1);
		return 1;
	}
 
	printf("reading hex file %s...\n", argv[1]);
	if (read_hex(&hf1)!=0)
	{
		printf("error in hex file %s\n", argv[1]);
		destroy_hexfile(&hf1);
		destroy_hexfile(&hf2);
		return 1;
	}

	printf("reading hex file %s...\n", argv[3]);
	if (read_hex(&hf2)!=0)
	{
		printf("error in hex file %s\n", argv[3]);
		destroy_hexfile(&hf1);
		destroy_hexfile(&hf2);
		return 1;
	}

	tf = fopen(argv[5], "w");
	if (tf==NULL)
	{
		printf("error creating hex file %s\n", argv[5]);
		return 1;
	}

	printf("writing hex file %s...\n", argv[5]);

	if (hf1.memsize>hf2.memsize)
		size = hf1.memsize;
	else
		size = hf2.memsize;
	for (addr=0; addr<size; addr+=0x10)
	{
		empty = 1;
		checksum = 0;
		for (pos=0; pos<0x10; pos++)
		{
			if (addr+pos>=startaddr2
				&& addr+pos<=endaddr2
				&& hf2.flag[addr+pos]==0)
			{
				linedata[pos] = hf2.mem[addr+pos];
				empty = 0;
			}
			else if (addr+pos>=startaddr1
				&& addr+pos<=endaddr1
				&& hf1.flag[addr+pos]==0)
			{
				linedata[pos] = hf1.mem[addr+pos];
				empty = 0;
			}
			else
				linedata[pos] = 0xFF;
			checksum += linedata[pos];
		}
		if (empty)
			continue;

		checksum += 0x10;
		checksum += (addr>>8);
		checksum += (addr&0xFF);
		checksum += 0x00;
		checksum = -checksum;

		fprintf(tf, ":10%04X00", addr);
		for (pos=0; pos<0x10; pos++)
			fprintf(tf, "%02X", linedata[pos]);
		fprintf(tf, "%02X\n", checksum);
	}

	fprintf(tf, ":00000001FF\n");

	fclose(tf);

	destroy_hexfile(&hf1);
	destroy_hexfile(&hf2);

	printf("done.\n");

	return 0;
}